"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AdvertisingIdPlugin = void 0;
var _analyticsReactNative = require("@segment/analytics-react-native");
var _reactNative = require("react-native");
class AdvertisingIdPlugin extends _analyticsReactNative.Plugin {
  type = _analyticsReactNative.PluginType.enrichment;
  queuedEvents = [];
  advertisingId = undefined;
  isLimitAdTracking = undefined;
  configure(analytics) {
    console.log('configure');
    if (_reactNative.Platform.OS !== 'android') {
      return;
    }
    this.analytics = analytics;
    this.fetchAdvertisingInfo().then(() => {
      // Additional logic after the advertising info is fetched
      this.sendQueued();
    }).catch(error => {
      this.handleError(error);
    });
  }
  async execute(event) {
    // If advertisingId is not set, queue the event
    if (this.advertisingId === undefined) {
      this.queuedEvents.push(event);
    } else {
      // Send event if advertisingId is available
      const currentLimitAdTrackingStatus = await this.fetchLimitAdTrackingStatus();
      if (this.isLimitAdTracking === undefined) {
        this.isLimitAdTracking = currentLimitAdTrackingStatus;
      } else if (this.isLimitAdTracking !== currentLimitAdTrackingStatus) {
        //Fetch the fresh advertising id
        await this.fetchAdvertisingInfo().then(() => {
          console.log('Advertising info fetched successfully when adTrackingStatus Changed.');
          // Additional logic after the advertising info is fetched
        }).catch(error => {
          this.handleError(error);
        });
        this.queuedEvents.push(event);
        this.isLimitAdTracking = currentLimitAdTrackingStatus;
        this.sendQueued();
        return;
      }
      return event;
    }
    return;
  }
  isTrackEvent(event) {
    return event.type === _analyticsReactNative.EventType.TrackEvent;
  }
  async fetchAdvertisingInfo() {
    const advertisingIdPromise = this.fetchAdvertisingId();
    const limitAdTrackingStatusPromise = this.fetchLimitAdTrackingStatus();
    try {
      // Await both promises to resolve simultaneously
      const [id, status] = await Promise.all([advertisingIdPromise, limitAdTrackingStatusPromise]);

      // Handle advertisingID
      if (id === null) {
        void this.analytics?.track('LimitAdTrackingEnabled (Google Play Services) is enabled');
        this.advertisingId = undefined; // Set to undefined if id is null
      } else {
        this.advertisingId = id;
      }

      // Set context after both values are available
      await this.setContext(id, status);
    } catch (error) {
      this.handleError(error);
    }
  }
  async setContext(id, isLimitAdTrackingEnableStatus) {
    try {
      await this.analytics?.context.set({
        device: {
          advertisingId: id,
          adTrackingEnabled: !isLimitAdTrackingEnableStatus
        }
      });
    } catch (error) {
      const message = 'AdvertisingID failed to set context';
      this.analytics?.reportInternalError(new _analyticsReactNative.SegmentError(_analyticsReactNative.ErrorType.PluginError, message, error));
      this.analytics?.logger.warn(`${message}: ${JSON.stringify(error)}`);
    }
  }
  sendQueued() {
    console.log('Sending queued events:', this.queuedEvents);
    this.queuedEvents.forEach(event => {
      void this.analytics?.process(event);
    });
    this.queuedEvents = [];
  }
  fetchAdvertisingId() {
    return (0, _analyticsReactNative.getNativeModule)('AnalyticsReactNativePluginAdvertisingId')?.getAdvertisingId();
  }
  fetchLimitAdTrackingStatus() {
    return (0, _analyticsReactNative.getNativeModule)('AnalyticsReactNativePluginAdvertisingId')?.getIsLimitAdTrackingEnableStatus();
  }
  handleError(error) {
    this.analytics?.reportInternalError(new _analyticsReactNative.SegmentError(_analyticsReactNative.ErrorType.PluginError, 'Error retrieving AdvertisingID', error));
  }
}
exports.AdvertisingIdPlugin = AdvertisingIdPlugin;
//# sourceMappingURL=AdvertisingIdPlugin.js.map